query RDBConnections {
  rdb {
    runs {
      edges {
        cursor
        node {
          id
          runNo
          state
          startedAt
          endedAt
          script
          exception
        }
      }
    }
    traces {
      edges {
        cursor
        node {
          id
          runNo
          traceNo
          startedAt
          endedAt
        }
      }
    }
    prompts {
      edges {
        cursor
        node {
          id
          runNo
          traceNo
          promptNo
          event
          startedAt
          fileName
          lineNo
        }
      }
    }
  }
}
